library IEEE;
use IEEE.STD_LOGIC_1164.ALL;	 
use IEEE.NUMERIC_STD.ALL;

entity ncc_top is  
	generic(	image_width : positive := 128;
				image_height : positive := 128;
				template_width : positive := 32; 
				template_height : positive := 32);
	port (
	   image : in integer range 0 to 255;  
	   template : in integer range 0 to 255; 
	   start : in std_logic;
       image_clk : in std_logic; 
	   clk : in std_logic;
       reset : in std_logic;

	   ready : out std_logic;
	   result_ready : out std_logic;
	   normxcorr : out integer;
	   pos_x : out integer range 0 to image_width - 1;
	   pos_y : out integer range 0 to image_height - 1
	);
end ncc_top;

architecture Behavioral of ncc_top is	  

COMPONENT sqrt
  PORT (
    aclk : IN STD_LOGIC;
    s_axis_cartesian_tvalid : IN STD_LOGIC;
    s_axis_cartesian_tdata : IN STD_LOGIC_VECTOR(47 DOWNTO 0);
    m_axis_dout_tvalid : OUT STD_LOGIC;
    m_axis_dout_tdata : OUT STD_LOGIC_VECTOR(31 DOWNTO 0)
  );
END COMPONENT;

------------- Begin Cut here for COMPONENT Declaration ------ COMP_TAG
COMPONENT divider
  PORT (
    aclk : IN STD_LOGIC;
    s_axis_divisor_tvalid : IN STD_LOGIC;
    s_axis_divisor_tready : OUT STD_LOGIC;
    s_axis_divisor_tdata : IN STD_LOGIC_VECTOR(39 DOWNTO 0);
    s_axis_dividend_tvalid : IN STD_LOGIC;
    s_axis_dividend_tready : OUT STD_LOGIC;
    s_axis_dividend_tdata : IN STD_LOGIC_VECTOR(47 DOWNTO 0);
    m_axis_dout_tvalid : OUT STD_LOGIC;
    m_axis_dout_tdata : OUT STD_LOGIC_VECTOR(47 DOWNTO 0)
  );
END COMPONENT;
-- COMP_TAG_END ------ End COMPONENT Declaration ------------

-- Configuration specification

	component convolution is
		generic(	image_width : positive := image_width;
					image_height : positive := image_height;
					template_height : positive := template_height;  
					template_width : positive := template_width);
		port (
			   image : in integer range 0 to 255;  
			   template : in integer range 0 to 255; 
			   start : in std_logic;
	           image_clk : in STD_LOGIC; 
			   clk : in STD_LOGIC;
	           reset : in STD_LOGIC;
			   
			   idle_indicator : out std_logic;
			   result : out integer range 0 to 2**31-1;
			   pos_x : out integer range 0 to image_width;
			   pos_y : out integer range 0 to image_height;
			   image_sum : out integer range 0 to 2**24-1;
			   image_sum_squared : out integer range 0 to 2**31-1;   
			   template_sum : out integer range 0 to 2**24-1;
			   template_sum_squared : out integer range 0 to 2**31-1;
			   result_ready : out std_logic); 
	end component;
	for all: convolution use entity work.convolution2(Behavioral);	
	
	signal square_root_in_ready_1 : std_logic;
	signal square_root_out_ready_1 : std_logic;	  
	signal square_root_data_in_1 : STD_LOGIC_VECTOR(47 DOWNTO 0);
	signal square_root_data_out_1 : STD_LOGIC_VECTOR(31 DOWNTO 0);	
	
	signal square_root_in_ready_2 : std_logic;
	signal square_root_out_ready_2 : std_logic;	  
	signal square_root_data_in_2 : STD_LOGIC_VECTOR(47 DOWNTO 0);
	signal square_root_data_out_2 : STD_LOGIC_VECTOR(31 DOWNTO 0);
	
	signal start_division : std_logic := '0';
	signal division_done : std_logic; 
	signal divisor_in : std_logic_vector(39 downto 0) := (others => '0');
	signal dividend_in : std_logic_vector(47 downto 0) := (others => '0');	  
	signal quotient_from_divider : std_logic_vector(47 downto 0);
	
	signal result_from_correlation : integer range 0 to 2**31-1;	  
	signal image_sum_from_correlation : integer range 0 to 2**24-1;
	signal image_sum_squared_from_correlation : integer range 0 to 2**31-1;		   
	signal template_sum_from_correlation : integer range 0 to 2**24-1;
	signal template_sum_squared_from_correlation : integer range 0 to 2**31-1;
	signal idle_indicator : std_logic; 	 
	signal ccorr_ready : std_logic;	 
	
	signal pos_x_from_correlation : integer range 0 to image_width;
	signal pos_y_from_correlation :integer range 0 to image_height;
	
	signal dividend : std_logic_vector (47 downto 0);
	
	type state is (IDLE, CCORR_CALCULATING, SQRT_CALCULATING_WAIT, 
		DIVISION_CALCULATING_WAIT, RESULT_OUTPUT);
	signal cur_state, next_state : state;	 
	
begin
	
	square_root_1 : sqrt
	  PORT MAP (
	    aclk => clk,
	    s_axis_cartesian_tvalid => square_root_in_ready_1,
	    s_axis_cartesian_tdata => square_root_data_in_1,
	    m_axis_dout_tvalid => square_root_out_ready_1,
	    m_axis_dout_tdata => square_root_data_out_1
	  );	 
	  
	  square_root_2 : sqrt
	  PORT MAP (
	    aclk => clk,
	    s_axis_cartesian_tvalid => square_root_in_ready_2,
	    s_axis_cartesian_tdata => square_root_data_in_2,
	    m_axis_dout_tvalid => square_root_out_ready_2,
	    m_axis_dout_tdata => square_root_data_out_2
	  );
	 
	  -- INST_TAG
	  divider_1 : divider
	  PORT MAP (
	    aclk => clk,
	    s_axis_divisor_tvalid => start_division,
	    s_axis_divisor_tready => open,
	    s_axis_divisor_tdata => divisor_in,
	    s_axis_dividend_tvalid => start_division,
	    s_axis_dividend_tready => open,
	    s_axis_dividend_tdata => dividend_in,
	    m_axis_dout_tvalid => division_done,
	    m_axis_dout_tdata => quotient_from_divider
	  );
	 -- INST_TAG_END
	 
	correlation : convolution
	generic map(
				image_width => image_width,
				image_height => image_height,
				template_height => template_height,  
				template_width => template_width
				)
	port map(
		   image , 
		   template , 
		   start ,
           image_clk ,
		   clk , 
           reset ,	
		   
		   idle_indicator,
		   result_from_correlation,	
		   pos_x_from_correlation,
		   pos_y_from_correlation,
		   image_sum_from_correlation,
		   image_sum_squared_from_correlation,
		   template_sum_from_correlation,
		   template_sum_squared_from_correlation,
		   ccorr_ready
		);	 

	calculation_process : process (clk) 
	variable denominator : std_logic_vector (47 downto 0);	   

	begin
		if (rising_edge(clk)) then 	 
			case (cur_state) is	  
				when CCORR_CALCULATING =>
					if (ccorr_ready = '1') then	
						dividend <= std_logic_vector( to_signed(template_width * template_height,16) * to_signed(result_from_correlation,32) 
						- to_signed(template_sum_from_correlation,24) * to_signed(image_sum_from_correlation,24));
					
						square_root_data_in_1 <= std_logic_vector( to_unsigned(template_width * template_height,16) * to_unsigned(image_sum_squared_from_correlation,32) 
							- to_unsigned(image_sum_from_correlation,24) * to_unsigned(image_sum_from_correlation,24));	 
						
						square_root_data_in_2 <= std_logic_vector( to_unsigned(template_width * template_height,16) * to_unsigned(template_sum_squared_from_correlation,32) 
							- to_unsigned(template_sum_from_correlation,24) * to_unsigned(template_sum_from_correlation,24));  
						
						square_root_in_ready_1  <= '1';	
						square_root_in_ready_2  <= '1';	
					end if;	  

				when SQRT_CALCULATING_WAIT => 	  
					square_root_in_ready_1  <= '0';	
					square_root_in_ready_2  <= '0';	 
					
					if (square_root_out_ready_1 = '1') then	
						denominator := std_logic_vector(resize((unsigned(square_root_data_out_1) * unsigned(square_root_data_out_2)), 48));	 

						if (denominator(47 downto 8) = std_logic_vector(TO_UNSIGNED(0,40)) ) then
							dividend_in <= std_logic_vector(TO_UNSIGNED(0,48));
							divisor_in <=  std_logic_vector(TO_UNSIGNED(1,40));
						else
							dividend_in <= dividend;
							divisor_in <=  denominator(47 downto 8);
						end if;
						
						start_division <= '1';

					end if;	 
				   
				when DIVISION_CALCULATING_WAIT =>   
					start_division <= '0';
				
					if (division_done = '1') then 
					
						normxcorr <= to_integer(signed(quotient_from_divider));
						result_ready <= '1'; 
					end if;

				when others =>	
					dividend <= (others => '0');
					
					normxcorr <= 0; 
					
					dividend_in <= (others => '0');
					divisor_in <=  (others => '0');
						
					start_division <= '0';
				
					square_root_data_in_1	<= (others => '0');
					square_root_data_in_2	<= (others => '0');
					square_root_in_ready_1 <= '0';	  
					square_root_in_ready_2 <= '0';	
					result_ready <= '0';
			end case;
		end if;
	end process calculation_process;
	
	ready_process : process (cur_state) 
	begin
		if (cur_state = IDLE) then
			ready <= '1';
		else 
			ready <= '0';
		end if;
	end process ready_process;
	
	result_process : process (clk)
	begin
		if (rising_edge(clk)) then 	
		end if;
	end process result_process;
	
	ncc_FSM_sync : process (clk)
	begin  
		if (rising_edge(clk)) then
			if (reset = '1') then
				cur_state <= IDLE;
			else
				cur_state <= next_state;
			end if;
		end if;
	end process ncc_FSM_sync;

	ncc_FSM_comb : process (cur_state, ccorr_ready, start, square_root_out_ready_1, idle_indicator, division_done)
	begin  
		case (cur_state) is
			when IDLE => 
			   	if (start = '1') then
					next_state <= CCORR_CALCULATING;
				else
					next_state <= IDLE;
				end if;		
				
			when CCORR_CALCULATING =>	
				if (idle_indicator = '1') then
					next_state <= IDLE;
				else
					if (ccorr_ready = '1') then
						next_state <= SQRT_CALCULATING_WAIT;
					else 
						next_state <= CCORR_CALCULATING;
					end if;	  
				end if;
			
			when SQRT_CALCULATING_WAIT =>
				if (square_root_out_ready_1 = '1') then
					next_state <= DIVISION_CALCULATING_WAIT;
				else 
					next_state <= SQRT_CALCULATING_WAIT;
				end if; 
			
			when DIVISION_CALCULATING_WAIT =>
				if (division_done = '1') then
					next_state <= RESULT_OUTPUT;
				else 
					next_state <= DIVISION_CALCULATING_WAIT;
				end if; 	
				
			when RESULT_OUTPUT =>
				next_state <= CCORR_CALCULATING;
			
		end case;		
	end process ncc_FSM_comb;
	
end Behavioral;