library IEEE;					
use IEEE.STD_LOGIC_1164.all;
use STD.TEXTIO.all;
use IEEE.NUMERIC_STD.ALL;

entity ncc_top_tb is
end ncc_top_tb;	 

architecture testbench of ncc_top_tb is   
	
	constant clkPeriod : time := 10 ns;
	constant image_clkPeriod : time := 400 ns;
		
component ncc_top is
	generic(	image_width : positive := 64;
				image_height : positive := 64;
				template_height : positive := 16;  
				template_width : positive := 16);
	port (
			   image : in integer range 0 to 255;  
			   template : in integer range 0 to 255; 
			   start : in std_logic;
		       image_clk : in std_logic; 
			   clk : in std_logic;
		       reset : in std_logic;
			   
			   ready : out std_logic;
			   result_ready : out std_logic;
			   
			   normxcorr : out integer;
			   
			   pos_x : out integer range 0 to image_width - 1;
			   pos_y : out integer range 0 to image_height - 1
			);
end component;
for all: ncc_top use entity work.ncc_top(Behavioral);	
  
	signal image : integer := 0;
	signal template : integer := 0;
	signal start : std_logic := '0';
    signal image_clk : std_logic := '0';
	signal clk : std_logic := '0'; 
    signal reset : std_logic := '0';		  
	signal pos_x : integer;
	signal pos_y : integer;
	signal result_ready : std_logic; 	 
	signal ready : std_logic;  
	
	signal normxcorr : integer;
	
	file IMAGEFILE : TEXT open READ_MODE is "image_64.txt";
	file TEMPLATEFILE : TEXT open READ_MODE is "template_16.txt";	 
	
	file RESULTFILE : TEXT open WRITE_MODE is "results.txt";	
	
	signal END_OF_SIMULATION : boolean := false;
	
begin	
	
	DUT : ncc_top
	generic map(
				image_width => 64,
				image_height => 64,
				template_height => 16,  
				template_width => 16
				)
	port map(
			   image,
			   template,
			   start,
		       image_clk ,
			   clk ,
		       reset ,
			   
			   ready,
			   result_ready ,
			   normxcorr,
			   pos_x ,
			   pos_y 
		);	 
	
	SYS_CLOCK: process
	begin
		if END_OF_SIMULATION = false 
			then
			clk <= '0';
			wait for clkPeriod/2;
			clk <= '1';
			wait for clkPeriod/2;
		else
			wait;
		end if;	
	end process;
	
	IMAGE_CLOCK: process
	begin
		if END_OF_SIMULATION = false 
			then
			image_clk <= '0';
			wait for image_clkPeriod/2;
			image_clk <= '1';
			wait for image_clkPeriod/2;
		else
			wait;
		end if;	
	end process;
	
	stimulate : process 

		variable IN_LINE : LINE;
		variable data_from_file : integer; 
		
	begin
		
		wait for clkPeriod;
		reset <= '1';
		wait for clkPeriod;	  
		reset <= '0';
		wait for clkPeriod;	  
		start <= '1';
		wait for clkPeriod;	  
		start <= '0';
		
		report "Loading template..." severity NOTE;

		while NOT(endfile(TEMPLATEFILE)) loop

		readline(TEMPLATEFILE,IN_LINE);
			read(IN_LINE,data_from_file);

			template <= data_from_file;
			
			wait for image_clkPeriod; 
			
		end loop;	   

		report "Done!" severity NOTE;
		report "Loading image..." severity NOTE; 
		
		while NOT(endfile(IMAGEFILE)) loop
			
			readline(IMAGEFILE,IN_LINE);
			read(IN_LINE,data_from_file);

			image <= data_from_file;
			wait for image_clkPeriod; 
			
		end loop;	
		
		report "Done!" severity NOTE; 
		
		wait on ready;
		END_OF_SIMULATION <= true;
		
		report "End of test vectors in file." severity warning;
		wait;
		
	end process;
	
	save_result : process (result_ready)
		variable line_num : line; 
		file file_pointer : text;
	begin	  		
		if (rising_edge(result_ready)) then
			write(line_num,normxcorr); --write the line.
	    	writeline (RESULTFILE,line_num); --write the contents into the file.	
 
		end if;
	end process save_result;
		
end architecture testbench;
