#ifdef __GNUC__
#  pragma GCC diagnostic ignored "-Wmissing-declarations"
#  if defined __clang__ || defined __APPLE__
#    pragma GCC diagnostic ignored "-Wmissing-prototypes"
#    pragma GCC diagnostic ignored "-Wextra"
#  endif
#endif

#include <ctime>
#include <cstdio>
#include <iostream>
#include <sys/time.h>
#include "cvconfig.h"
#include <cuda.h>
#include <cuda_runtime.h>
#include "opencv2/core/core.hpp"
#include "opencv2/core/opengl_interop.hpp"
#include "opencv2/highgui/highgui.hpp"
#include "opencv2/imgproc/imgproc.hpp"
#include "opencv2/ts/ts.hpp"
#include "opencv2/ts/ts_perf.hpp"
#include "opencv2/gpu/gpu.hpp"

using namespace cv;
using namespace cv::gpu;
using namespace std;
using namespace cvtest;
using namespace testing;
using namespace testing::internal;

int randomInt(int minVal, int maxVal) {
    RNG& rng = TS::ptr()->get_rng();
    return rng.uniform(minVal, maxVal);
}

Mat readImage(const std::string& fileName, int flags) {
    return imread(TS::ptr()->get_data_path() + fileName, flags);
}

GpuMat createMat(Size size, int type) {
    Size size0 = size;

    GpuMat d_m(size0, type);

    if (size0 != size)
        d_m = d_m(Rect((size0.width - size.width) / 2, (size0.height - size.height) / 2, size.width, size.height));

    return d_m;
}

GpuMat loadMat(const Mat& m) {
    GpuMat d_m = createMat(m.size(), m.type());
    d_m.upload(m);
    return d_m;
}

void reportGPUTime(int kernelSize, int sceneSize){

	if (!(sceneSize >= kernelSize)){
		return;
	}

	cv::Mat image(sceneSize, sceneSize, CV_8UC3);
	cv::Mat pattern(kernelSize, kernelSize, CV_8UC3);
	
	randn(image, Scalar::all(128), Scalar::all(127));
	randn(pattern, Scalar::all(128), Scalar::all(127));

	int method = CV_TM_CCOEFF_NORMED;
	cv::gpu::GpuMat d_dst;
	
	cudaEvent_t start, stop;
	float time;
	cudaEventCreate(&start);
	cudaEventCreate(&stop);
	cudaEventRecord(start, 0);

	cv::gpu::matchTemplate(loadMat(image), loadMat(pattern), d_dst, method);

	cudaEventRecord(stop, 0);
	cudaEventSynchronize(stop);
	cudaEventElapsedTime(&time, start, stop);
	cudaEventDestroy(start);
	cudaEventDestroy(stop);
	
	cv::Mat dst(d_dst);
    double maxValue;
    cv::Point maxLoc;
    cv::minMaxLoc(dst, NULL, &maxValue, NULL, &maxLoc);
	
	printf("0, %d, %d, %f, %d, %d\n", kernelSize, sceneSize, time, maxLoc.x, maxLoc.x);
}

void initializeCUDA() {
	int method = CV_TM_CCORR_NORMED;
	cv::Mat image = readImage("matchtemplate/black.png");
	cv::Mat pattern = readImage("matchtemplate/cat.png");    
	cv::Mat d_dst_cpu;
	cv::gpu::GpuMat d_dst;
    cv::gpu::matchTemplate(loadMat(image), loadMat(pattern), d_dst, method);
    cv::matchTemplate(image, pattern, d_dst_cpu, method);
}

int main( int argc, char** argv ) {
	
	#if !defined(HAVE_CUDA)
		std::cout << "CUDA support is required (CMake key 'WITH_CUDA' must be true).\n";
		return 1;
	#endif

	initializeCUDA();
	printf("device, kernel side, image side, time, result\n");
	
	for (int m = 1; m <= 50; m++){
			for (int i = 1; i <= 500; i++){
				reportGPUTime(i, 32*m);
			}
	}
	return 0;
}


